"""
テストランに登録されたテスト結果の添付ファイルを取得します。

使い方：
１．TestRail の [管理] > [サイト設定] > [API] タブで 'API の有効化' チェックボックスを有効化してください。
２．ログイン情報をご利用環境に合わせて変更してください。
３．実行時引数を2つ指定してください。
　　・第一引数：テストランID
　　・第二引数：ファイルダウンロード先フォルダーパス (事前に用意してください。)
    実行例：　python.exe get_attachment_run.py 5 C:\\sample
    ↑テストランID が 5 のテスト結果に添付されたファイルを C:\\sample にダウンロードします。
4. 指定したフォルダーパスに 'テストID\結果ID_ステータス' のフォルダーを作成してファイルをダウンロードします。

出力されるファイル：
1. テスト結果に添付されたファイル
2. コメント欄等に埋め込まれた画像ファイル
   TestRailが自動で採番したIDをファイル名としてダウンロードします。
   どこに埋め込まれた画像であるかは、「3．テスト結果」を確認してください。
3. テスト結果(result.json)

その他：
1. 全てのテスト結果の添付ファイルを取得します。
   'Passed' や 'Failed' などのステータスや、日付で絞込みを行う場合 'get_results_for_run' のリクエストフィルターを使用してください。
   'get_results_for_run' の使用方法は以下のマニュアルを参照してください。
   https://docs.testrail.techmatrix.jp/api/reference/results/#get_results_for_run

"""

import sys
import os
import re
import json
import codecs
from testrail import *

# 実行時引数の配列
# 第一引数：テストランID
# 第二引数：フォルダーパス
args = sys.argv

# TestRail ログイン情報 (★ ご利用環境に合わせて変更してください。)
client = APIClient('http://localhost:18888/testrail/')
client.user = 'taro@techmatrix.co.jp'
client.password = 'taro'

# テストランに含まれるテスト結果情報を取得 (★ 'get_results_for_run' のリクエストフィルター)
dict_result_run = client.send_get('get_results_for_run/' + args[1] )
print(str(len(dict_result_run)) + " 件のテスト結果があります。")

# ステータスが 'Passed' のテスト結果から添付ファイルを取得する
# dict_result_run = client.send_get('get_results_for_run/' + args[1] + '&status_id=1')

# ステータス情報の取得
status_information = client.send_get('get_statuses')

# ステータス情報の ID と ラベルでステータスの辞書を作成
status_dict = {}
for one_status in status_information:
    status_dict[one_status.get('id')] = one_status.get('label')

# テスト結果の数分繰り返す
for i, one_result in enumerate(dict_result_run):
    # 結果ID
    result_id = one_result.get('id')

    # テストID
    test_id = one_result.get('test_id')

    print(str((i + 1)) + " 件目を処理しています。テストID " + str(test_id) + "、テスト結果ID " + str(result_id))

    # ステータスID
    result_status_id = one_result.get('status_id')

    # ステータスラベルを取得。ステータスIDが存在しない場合は Comment 扱い。
    if result_status_id in status_dict:
        result_status_label = status_dict[result_status_id]
    else:
        result_status_label = 'Comment'

    # ダウンロード先フォルダー
    download_folder = args[2] + '\\' + str(test_id) + '\\' + str(result_id) + '_' + result_status_label

    # テスト結果をJSON ⇒ 辞書型 ⇒ 文字列型に変換
    one_result_dict = json.dumps(one_result)
    one_result_str = json.dumps(one_result_dict)

    # 埋め込み画像情報を取得
    embedded_image_list = re.findall(r'index.php\?/attachments/get/[0-9]{1,}', one_result_str)

    # 埋め込み画像のID取得用
    embedded_image_list_id = []

    # 埋め込む画像数分繰り返す
    for one_embedded_image in embedded_image_list:
        # フォルダーがある場合は作成しない
        if not os.path.isdir(download_folder):
            os.makedirs(download_folder)

        # ファイル取得用API
        embedded_request_api = one_embedded_image.replace('index.php?/attachments/get/', 'get_attachment/')

        # 埋め込み画像の ID
        embedded_image_id = embedded_request_api.replace('get_attachment/', '')

        # ファイル名作成
        embedded_image_file = embedded_image_id + ".png"

        # 埋め込み画像の ID をリストに追加
        embedded_image_list_id.append(str(embedded_image_id))

        # ファイルパス
        embedded_download_file_path = download_folder + '\\' + embedded_image_file

        # ファイルをダウンロード
        result = client.send_get(embedded_request_api, embedded_download_file_path)

    # テストの添付ファイルのリストを取得 (リストには複数回のテスト結果に添付されたファイル情報が含まれる)
    list_attachment = client.send_get('get_attachments_for_test/' + str(test_id))

    # 添付ファイル数分繰り返す
    for one_attachment in list_attachment:

        # 結果IDが同じ場合だけ取得する
        if (str(result_id) == str(one_attachment.get('result_id'))):

            # フォルダーがある場合は作成しない
            if not os.path.isdir(download_folder):
                os.makedirs(download_folder)

            # ファイル名
            file_name = one_attachment.get('name')

            # ファイルID
            file_id = one_attachment.get('id')

            # ファイル ID が埋め込み画像の ID と一致する場合は取得しない
            if not str(file_id) in embedded_image_list_id:
                # ファイル取得用API
                request_api = 'get_attachment/' + str(file_id)

                # ファイルパス
                download_file_path = download_folder + '\\' + file_name

                # ファイルをダウンロード
                result = client.send_get(request_api, download_file_path)

    # フォルダーがある場合はテスト結果をファイル出力する
    if os.path.isdir(download_folder):
        # テスト結果をファイル出力
        result_file_path = download_folder + '\\result.json'

        with codecs.open(result_file_path, 'w', "utf-8") as f:
            json.dump(one_result, f, ensure_ascii=False, indent=4)
